@@ -4,11 +4,12 @@ use crate::{
     fmt_symbol,
     formatters::{
         assignment::hang_punctuated_list,
-        expression::format_expression,
-        general::{format_symbol, try_format_punctuated},
+        expression::{format_expression, hang_expression},
+        general::{format_punctuated, format_punctuated_multiline, format_symbol},
         stmt::format_stmt,
         trivia::{
-            strip_trivia, FormatTriviaType, UpdateLeadingTrivia, UpdateTrailingTrivia, UpdateTrivia,
+            strip_trailing_trivia, strip_trivia, FormatTriviaType, UpdateLeadingTrivia,
+            UpdateTrailingTrivia, UpdateTrivia,
         },
         trivia_util,
     },
@@ -19,8 +20,6 @@ use full_moon::ast::{
 };
 use full_moon::tokenizer::TokenType;
 use full_moon::tokenizer::{Token, TokenReference};
-#[cfg(feature = "luau")]
-use std::borrow::Cow;
 
 macro_rules! update_first_token {
     ($enum:ident, $var:ident, $token:expr, $update_method:ident) => {{
@@ -30,11 +29,7 @@ macro_rules! update_first_token {
     }};
 }
 
-pub fn format_return<'ast>(
-    ctx: &Context,
-    return_node: &Return<'ast>,
-    shape: Shape,
-) -> Return<'ast> {
+pub fn format_return(ctx: &Context, return_node: &Return, shape: Shape) -> Return {
     // Calculate trivia
     let leading_trivia = vec![create_indent_trivia(ctx, shape)];
     let trailing_trivia = vec![create_newline_trivia(ctx)];
@@ -51,35 +46,92 @@ pub fn format_return<'ast>(
             .update_leading_trivia(FormatTriviaType::Append(leading_trivia));
 
         let shape = shape + (strip_trivia(return_node.token()).to_string().len() + 1); // 1 = " "
-        let mut formatted_returns = try_format_punctuated(
-            ctx,
-            return_node.returns(),
-            shape,
-            format_expression,
-            Some(1),
+
+        let returns = return_node.returns();
+
+        let contains_comments = trivia_util::contains_comments(
+            returns.update_trailing_trivia(FormatTriviaType::Replace(Vec::new())), // We can ignore trailing trivia, as that won't affect anything
         );
 
-        // Determine if we need to hang the condition
-        let require_multiline_expression = shape
-            .take_first_line(&strip_trivia(&formatted_returns))
-            .over_budget()
-            || {
-                trivia_util::contains_comments(
-                    return_node
-                        .returns()
-                        .update_trailing_trivia(FormatTriviaType::Replace(Vec::new())), // We can ignore trailing trivia, as that won't affect anything
-                )
-            };
+        // See if we need to format multiline
+        // If we contain comments, we immediately force multiline, and return an empty Punctuated sequence as a placeholder (it will never be used)
+        // If not, format the sequence on a single line, and test the shape. We return the singleline output incase we want to use it.
+        // We do it this way so that the singleline return is evaluated lazily - we don't want to create it if we never use it, but if we
+        // create it, we need to keep it incase we want to use it.
+        let (should_format_multiline, singleline_returns) = if contains_comments {
+            (true, Punctuated::new())
+        } else {
+            // Firstly attempt to format the returns onto a single line, using an infinite column width shape
+            let singleline_returns =
+                format_punctuated(ctx, returns, shape.with_infinite_width(), format_expression);
 
-        if require_multiline_expression {
-            formatted_returns = hang_punctuated_list(ctx, return_node.returns(), shape);
-        }
+            // Test the return to see if its over width
+            let singleline_shape =
+                shape + strip_trailing_trivia(&singleline_returns).to_string().len();
+            (singleline_shape.over_budget(), singleline_returns)
+        };
 
-        if let Some(pair) = formatted_returns.pop() {
-            let pair = pair
-                .map(|expr| expr.update_trailing_trivia(FormatTriviaType::Append(trailing_trivia)));
-            formatted_returns.push(pair);
-        }
+        // TODO: this is similar to assignment tactics - can we abstract them into a common function?
+        let formatted_returns = if should_format_multiline {
+            if returns.len() > 1 {
+                // Format the punctuated onto multiple lines
+                let hang_level = Some(1);
+                let multiline_returns =
+                    format_punctuated_multiline(ctx, returns, shape, format_expression, hang_level);
+
+                let mut output_returns = Punctuated::new();
+
+                // Look through each punctuated sequence to see if we need to hang the item further
+                for (idx, (formatted, original)) in
+                    multiline_returns.into_pairs().zip(returns).enumerate()
+                {
+                    // Recreate the shape
+                    let shape = if idx == 0 {
+                        shape
+                    } else {
+                        shape
+                            .reset()
+                            .with_indent(shape.indent().add_indent_level(hang_level.unwrap()))
+                    };
+
+                    if trivia_util::contains_comments(&formatted)
+                        || shape.take_first_line(&formatted).over_budget()
+                    {
+                        // Hang the pair, using the original expression for formatting
+                        output_returns
+                            .push(formatted.map(|_| hang_expression(ctx, original, shape, Some(1))))
+                    } else {
+                        // Add the pair as it is
+                        output_returns.push(formatted);
+                    }
+                }
+
+                output_returns
+            } else {
+                // Create an example hanging the expression - we need to create a new context so that we don't overwrite it
+                let hanging_returns = hang_punctuated_list(ctx, returns, shape);
+                let hanging_shape = shape.take_first_line(&strip_trivia(&hanging_returns));
+
+                // Create an example formatting the expression normally
+                let formatted_returns = format_punctuated(ctx, returns, shape, format_expression);
+                let formatting_shape =
+                    shape.take_first_line(&strip_trailing_trivia(&formatted_returns));
+
+                // Find the better format out of the hanging shape or the normal formatting
+                if hanging_shape.used_width() < formatting_shape.used_width() {
+                    // Hanging version is better
+                    hanging_returns
+                } else {
+                    formatted_returns
+                }
+            }
+        } else {
+            singleline_returns
+        };
+
+        // Add a newline at the end of the list
+        let formatted_returns =
+            formatted_returns.update_trailing_trivia(FormatTriviaType::Append(trailing_trivia));
 
         Return::new()
             .with_token(token)
@@ -87,11 +139,7 @@ pub fn format_return<'ast>(
     }
 }
 
-pub fn format_last_stmt<'ast>(
-    ctx: &Context,
-    last_stmt: &LastStmt<'ast>,
-    shape: Shape,
-) -> LastStmt<'ast> {
+pub fn format_last_stmt(ctx: &Context, last_stmt: &LastStmt, shape: Shape) -> LastStmt {
     check_should_format!(ctx, last_stmt);
 
     match last_stmt {
@@ -111,7 +159,7 @@ pub fn format_last_stmt<'ast>(
                 &TokenReference::new(
                     vec![],
                     Token::new(TokenType::Identifier {
-                        identifier: Cow::Owned(String::from("continue")),
+                        identifier: "continue".into(),
                     }),
                     vec![],
                 ),
@@ -127,7 +175,7 @@ pub fn format_last_stmt<'ast>(
     }
 }
 
-fn trivia_remove_leading_newlines<'ast>(trivia: Vec<&Token<'ast>>) -> Vec<Token<'ast>> {
+fn trivia_remove_leading_newlines(trivia: Vec<&Token>) -> Vec<Token> {
     trivia
         .iter()
         .skip_while(|x| match x.token_type() {
@@ -138,7 +186,7 @@ fn trivia_remove_leading_newlines<'ast>(trivia: Vec<&Token<'ast>>) -> Vec<Token<
         .collect()
 }
 
-fn prefix_remove_leading_newlines<'ast>(prefix: &Prefix<'ast>) -> Prefix<'ast> {
+fn prefix_remove_leading_newlines(prefix: &Prefix) -> Prefix {
     match prefix {
         Prefix::Name(token) => {
             let leading_trivia = trivia_remove_leading_newlines(token.leading_trivia().collect());
@@ -315,14 +363,17 @@ fn last_stmt_remove_leading_newlines(last_stmt: LastStmt) -> LastStmt {
 }
 
 /// Formats a block node. Note: the given shape to the block formatter should already be at the correct indentation level
-pub fn format_block<'ast>(ctx: &Context, block: &Block<'ast>, shape: Shape) -> Block<'ast> {
-    let mut formatted_statements: Vec<(Stmt<'ast>, Option<TokenReference<'ast>>)> = Vec::new();
+pub fn format_block(ctx: &Context, block: &Block, shape: Shape) -> Block {
+    let mut ctx = *ctx;
+    let mut formatted_statements: Vec<(Stmt, Option<TokenReference>)> = Vec::new();
     let mut found_first_stmt = false;
     let mut stmt_iterator = block.stmts_with_semicolon().peekable();
 
     while let Some((stmt, semi)) = stmt_iterator.next() {
+        ctx = ctx.check_toggle_formatting(stmt);
+
         let shape = shape.reset();
-        let mut stmt = format_stmt(ctx, stmt, shape);
+        let mut stmt = format_stmt(&ctx, stmt, shape);
 
         // If this is the first stmt, then remove any leading newlines
         if !found_first_stmt {
@@ -371,7 +422,7 @@ pub fn format_block<'ast>(ctx: &Context, block: &Block<'ast>, shape: Shape) -> B
                 stmt = updated_stmt;
                 Some(
                     match semi {
-                        Some(semi) => crate::fmt_symbol!(ctx, semi, ";", shape),
+                        Some(semi) => crate::fmt_symbol!(&ctx, semi, ";", shape),
                         None => TokenReference::symbol(";").expect("could not make semicolon"),
                     }
                     .update_trailing_trivia(FormatTriviaType::Append(trivia)),
@@ -386,7 +437,7 @@ pub fn format_block<'ast>(ctx: &Context, block: &Block<'ast>, shape: Shape) -> B
                     // We will do a hack here, where we insert an empty token, and add all the remaining trivia onto it
                     Some(
                         format_symbol(
-                            ctx,
+                            &ctx,
                             semi,
                             &TokenReference::new(vec![], Token::new(TokenType::spaces(0)), vec![]),
                             shape,
@@ -406,8 +457,10 @@ pub fn format_block<'ast>(ctx: &Context, block: &Block<'ast>, shape: Shape) -> B
 
     let formatted_last_stmt = match block.last_stmt_with_semicolon() {
         Some((last_stmt, semi)) => {
+            ctx = ctx.check_toggle_formatting(last_stmt);
+
             let shape = shape.reset();
-            let mut last_stmt = format_last_stmt(ctx, last_stmt, shape);
+            let mut last_stmt = format_last_stmt(&ctx, last_stmt, shape);
             // If this is the first stmt, then remove any leading newlines
             if !found_first_stmt && ctx.should_format_node(&last_stmt) {
                 last_stmt = last_stmt_remove_leading_newlines(last_stmt);
@@ -423,7 +476,7 @@ pub fn format_block<'ast>(ctx: &Context, block: &Block<'ast>, shape: Shape) -> B
                     // We want to keep any old comments on the semicolon token, otherwise we will lose it
                     // We will do a hack here, where we replace the semicolon with an empty symbol
                     let semicolon_token = format_symbol(
-                        ctx,
+                        &ctx,
                         semi,
                         &TokenReference::new(vec![], Token::new(TokenType::spaces(0)), vec![]),
                         shape,
